"""
    spout
    ====

    A simple framework that makes it easy to work with data streams in Python.

    :copyright: (c) 2014 by Jamie Davies.
    :license: MIT, see LICENSE for more details.
"""

__version__ = '0.1.5'
__author__ = 'Jamie Davies'
